<?php
add_action( 'dwd_wc_validate_attributes_cron', 'dwd_wc_mark_invalid_products' );
if ( ! wp_next_scheduled( 'dwd_wc_validate_attributes_cron' ) ) {
	wp_schedule_event( time(), 'twicedaily', 'dwd_wc_validate_attributes_cron' );
}

/**
 * @return WC_Product[]
 */
function dwd_wc_get_invalid_products() {
	$products         = dwd_wc_get_linked_products();
	$invalid_products = [];

	$api = DWD_WC_API_Helper::getInstance();

	foreach ( $products as $product ) {
		$dwd_product_data = dwd_wc_get_product_data( $product );
		$dwd_attributes   = $api->getAvailableOptions( $dwd_product_data['uuid'], [] );
		$dwd_ranges       = $api->getAvailableRanges( $dwd_product_data['uuid'], [] );
		$wc_attributes    = $dwd_product_data['attributes'];

		// validate attribute keys first
		foreach ( array_keys( $wc_attributes ) as $wc_attribute ) {
			if ( ! array_key_exists( $wc_attribute, $dwd_attributes ) && ! array_key_exists( $wc_attribute, $dwd_ranges ) ) {
				$invalid_products[] = $product;
				break 2;
			}
		}

		// next, validate values (only for regular attributes, skip ranges
		foreach ( $wc_attributes as $wc_attribute => $wc_value ) {
			if ( array_key_exists( $wc_attribute, $dwd_ranges ) ) {
				continue;
			}

			if ( ! in_array( $wc_value, $dwd_attributes[ $wc_attribute ] ) ) {
				$invalid_products[] = $product;
				break 2;
			}
		}
	}

	return $invalid_products;
}

function dwd_wc_mark_invalid_products() {
	$invalid_products = dwd_wc_get_invalid_products();
	if ( count( $invalid_products ) === 0 ) {
		delete_transient( 'dwd-wc-invalid-products' );

		return;
	}

	$invalid_product_names = [];
	foreach ( $invalid_products as $product ) {
		$invalid_product_names[ $product->get_id() ] = $product->get_name();
	}

	set_transient( 'dwd-wc-invalid-products', $invalid_product_names );
}

function dwd_wc_get_dwd_attributes( $uuid ) {
	$api                  = DWD_WC_API_Helper::getInstance();
	$available_attributes = $api->getAvailableOptions( $uuid, [] );

	return $available_attributes;
}

add_action( 'admin_notices', function () {
	$invalid_products = get_transient( 'dwd-wc-invalid-products' );
	if ( ! $invalid_products ) {
		return;
	}
	?>
    <div class="notice notice-warning">
        <p>The following products are no longer valid and cannot be submitted to Drukwerkdeal:</p>
        <ul class="list ul-disc">
			<?php foreach ( $invalid_products as $id => $name ): ?>
                <li><?php echo "#$id: $name"; ?></li>
			<?php endforeach; ?>
        </ul>
    </div>
	<?php
} );