const SELECTOR_CREATED = "SelectorCreatedEvent";
const SELECTOR_UPDATED = "SelectorUpdatedEvent";

(function($) {
    console.log(dwd_wc);
    $(document).on("change", 'select.dwd_product_uuid', async (event) => {
        const $select = $(event.target);
        const loop = $select.data('loop');
        const uuid = $select.val();
        if (uuid) {
            initSelector(loop, uuid, false);
        }
    });

    $(document).on("change", ".dwd-product-selector-attribute", async (event) => {
        const loop = $(event.target).data('loop');

        const $select = $(`select#dwd_product_uuid${loop}`);
        const uuid = $select.val();
        await validateSelector(loop, uuid);
        updateProductJSON(loop);
        document.dispatchEvent(new CustomEvent(SELECTOR_UPDATED, {detail: {loop}}));
    });

    document.addEventListener(SELECTOR_UPDATED, (e) => {
        const loop = e.detail.loop;
        const tab = document.querySelector(`#dwd-product-select-${loop}`).parentNode;
        const priceRow = tab.querySelector('.price-row');
        if (priceRow) {
            tab.removeChild(priceRow);
        }
        if (isFullySelected(loop)) {
            showPrice(loop);
        }
    })
})(jQuery);

async function initSelector(loop, uuid, useCurrent) {
    // init vars and clear selector
    const $ = jQuery;
    $(".dwd-wc-selector-attribute-dropdown").remove();
    const dwdJson = $(`#dwd-json-${loop}`).val();
    let currentSelection = {};
    try {
        currentSelection = JSON.parse(dwdJson);
    } catch (e) {
        console.error("Could not parse", dwdJson);
    }
    const $productSelect = $(`#dwd-product-select-${loop}`);
    const dropdowns = await createDropDowns(loop, uuid, useCurrent, currentSelection);
    const inputs = await createInputs(loop, uuid, useCurrent, currentSelection);

    $productSelect.after(inputs);
    $productSelect.after(dropdowns);
    document.dispatchEvent(new CustomEvent(SELECTOR_CREATED, {detail: {loop}}));

    await validateSelector(loop, uuid);
    document.dispatchEvent(new CustomEvent(SELECTOR_UPDATED, {detail: {loop}}));
}

async function createDropDowns(loop, uuid, useCurrent, currentSelection) {
    const $ = jQuery;

    // retrieve all available options for this product
    const url = `${dwd_wc.DWD_WC_WRAPPER_URL}/available-options`;
    const params = {
        uuid
    };
    const attributes = await $.ajax({
        url: `${url}?${(new URLSearchParams(params)).toString()}`,
        method: 'get',
        headers: dwd_wc.DWD_WC_API_HEADERS
    });
    // for each attribute, add a select with options
    let divs = "";
    for (const attribute of Object.keys(attributes)) {
        let div = `
            <div class="options_group dwd-wc-selector-attribute-dropdown">
                <div>
                    <div class="options_group">
                        <p class="form-row form-row-full form-field">
                            <label>${attribute}</label>
                            <select 
                                name="selector-${loop}-${attribute}" 
                                class="dwd-product-selector-attribute dwd-product-selector-attribute-${loop}" 
                                data-attribute="${attribute}" 
                                data-loop="${loop}"
                            >
                                <option></option>
            `;
        for (const value of attributes[attribute]) {
            if (useCurrent && currentSelection[attribute] && currentSelection[attribute] === value) {
                div += `<option selected="selected">${value}</option>`;
            } else {
                div += `<option>${value}</option>`;
            }
        }
        div += `
                            </select>
                        </p>
                    </div>
                </div>
            </div>
        `;
        divs += div;
    }
    return divs;
}

async function createInputs(loop, uuid, useCurrent, currentSelection) {
    const $ = jQuery;
    const params = {
        uuid
    };

    const url = `${dwd_wc.DWD_WC_WRAPPER_URL}/available-ranges`;
    const ranges = await $.ajax({
        url: `${url}?${(new URLSearchParams(params)).toString()}`,
        method: 'get',
        headers: dwd_wc.DWD_WC_API_HEADERS
    });

    let divs = "";
    for (const attribute of Object.keys(ranges)) {
        let div = `
            <div class="options_group dwd-wc-selector-attribute-dropdown">
                <div>
                    <div class="options_group">
                        <p class="form-row form-row-full form-field">
                            <label>${attribute} (${ranges[attribute].min}-${ranges[attribute].max})</label>
                            <input 
                                name="selector-${loop}-${attribute}" 
                                class="dwd-product-selector-range dwd-product-selector-attribute dwd-product-selector-attribute-${loop}" 
                                data-attribute="${attribute}"
                                data-loop="${loop}"
                                type="number"
                                min="${ranges[attribute].min}"
                                max="${ranges[attribute].max}"
                                step="${ranges[attribute].inc}"
                                value="${useCurrent && currentSelection ? currentSelection[attribute] : ''}"
                            >
                        </p>
                    </div>
                </div>
            </div>
        `;
        divs += div;
    }
    return divs;
}

async function validateSelector(loop, uuid) {
    const $ = jQuery;
    const selects = $(`.dwd-product-selector-attribute-${loop}`);
    const params = {
        uuid
    }
    for (const s of selects) {
        const select = $(s);
        if (select.val()) {
            params[select.data('attribute')] = select.val();
        }
    }

    // retrieve all available options for this product
    const url = `${dwd_wc.DWD_WC_WRAPPER_URL}/available-options`;
    const response = await $.ajax({
        url: `${url}?${(new URLSearchParams(params)).toString()}`,
        method: 'get',
        headers: dwd_wc.DWD_WC_API_HEADERS
    });

    for (const s of selects) {
        const select = $(s);
        const attribute = select.data("attribute");
        const options = select.find("option");
        for (const option of options) {
            const $option = $(option);
            const value = option.value;
            if (!value) {
                continue;
            }

            if (!response[attribute].includes(value)) {
                $option.attr('disabled', 'disabled');
            } else {
                $option.removeAttr('disabled');
            }
        }
        if (select.val() === "") {
            autoSelect(select);
        }
    }
}

function isFullySelected(loop) {
    const attributes = document.querySelectorAll(`.dwd-product-selector-attribute-${loop}`);
    const emptyAttributes = Array.from(attributes).filter(a => a.value === undefined || a.value.length <= 0);
    return emptyAttributes.length <= 0;
}

async function showPrice(loop) {
    const product = getDwdProduct(loop);
    const div =
        `<div class="options_group price-row">
        <div>
            <div class="options_group">
                <p class="form-row form-row form-field" id="dwd-price-${loop}">
                    Loading price... <span class="spinner is-active" style="float: none;">
                </p>
            </div>
        </div>
    </div>`
    // find the parent tab, and from there the upload file element
    const tab = document.querySelector(`#dwd-product-select-${loop}`).parentNode;
    const fileUpload = tab.querySelector(':scope >div.options_group:last-child');

    // add the price before the file element
    fileUpload.insertAdjacentHTML('beforebegin', div);

    const result = await wp.ajax.post("dwd_wc_get_product_price", {product: product});
    console.log(result);
    const price = Intl.NumberFormat(undefined, {style: "currency", currency: "EUR"}).format(result.price);
    document.querySelector(`#dwd-price-${loop}`).innerHTML = `Starting price: ${price}. Next delivery date: ${result.delivery_date}.`
}

function autoSelect(select) {
    const $ = jQuery;
    const availableOptions = [];
    for (const o of select.find("option")) {
        const option = $(o);
        if (option.attr('disabled') || !o.value) {
            continue;
        }
        availableOptions.push(option);
    }

    if (availableOptions.length === 1) {
        select.val(availableOptions[0].val());
    }
}

function getDwdProduct(loop) {
    const attributes = {};
    for (const attribute of document.querySelectorAll(`.dwd-product-selector-attribute-${loop}`)) {
        if (attribute.value) {
            attributes[attribute.getAttribute('data-attribute')] = attribute.value;
        }
    }
    return {
        uuid: document.querySelector(`select#dwd_product_uuid${loop}`).value,
        attributes
    }
}

function updateProductJSON(loop) {
    const $ = jQuery;
    const jsonField = $(`#dwd-json-${loop}`);
    const json = {};
    for (const attribute of $(`.dwd-product-selector-attribute-${loop}`)) {
        const $attribute = $(attribute);
        if ($attribute.val()) {
            json[$attribute.data('attribute')] = $attribute.val();
        }
    }

    jsonField.val(JSON.stringify(json));
}