<?php

class DWD_WC_State_Machine {
	private static $instance;

	protected $dwd_happy_states = [
		'waiting-for-artwork',
		'in-progress',
		'in-production',
		'shipped',
		'cancelled'
	];

	protected $dwd_problem_states = [
		'unknown',
		'artwork-rejected',
	];

	protected $wc_states = [
		'waiting-for-artwork' => 'wc-dwd-wfa',
		'in-progress' => 'wc-dwd-progress',
		'in-production' => 'wc-dwd-printing',
		'shipped' => 'wc-dwd-shipped',
		'cancelled' => 'wc-dwd-cancelled',
		'unknown' => 'wc-dwd-unknown',
		'artwork-rejected' => 'wc-dwd-artwork-rejected'
	];

	public static function getInstance() {
		if (!self::$instance) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	private function __construct()
	{
	}

	public function get_wc_statuses()
	{
		return array_values($this->wc_states);
	}

	public function get_wc_state($lineitems)
	{
		// we take the lowest lineitem state as the one we display. So if just one
		// of your lineitems is wfa the entire order is considered to be wfa.
		$dwd_state = null;
		if ($this->has_problem_state($lineitems)) {
			$dwd_state = $this->get_problem_state($lineitems);
		} else {
			$dwd_state = $this->get_happy_state($lineitems);
		}
		return $this->wc_states[$dwd_state];
	}

	protected function has_problem_state($lineitems)
	{
		foreach ($lineitems as $lineitem) {
			if (in_array($lineitem['status'], $this->dwd_problem_states)) {
				return true;
			}
		}
		return false;
	}

	protected function get_problem_state($lineitems)
	{
		return $this->get_state_from_states($lineitems, $this->dwd_problem_states);
	}

	protected function get_happy_state($lineitems)
	{
		return $this->get_state_from_states($lineitems, $this->dwd_happy_states);
	}

	protected function get_state_from_states($lineitems, $states)
	{
		$state = end($states);
		foreach ($lineitems as $lineitem)
		{
			if (!in_array($lineitem['status'], $states)) {
				continue;
			}

			$state = $this->get_lowest_state($lineitem['status'], $state, $states);
		}
		return $state;
	}

	protected function get_lowest_state($state1, $state2, $states) {
		$state1_pos = array_search($state1, $states);
		$state2_pos = array_search($state2, $states);
		if ($state1_pos < $state2_pos) {
			return $state1;
		}
		return $state2;
	}
}